@@ -1,6 +1,13 @@
 jomo1con<- function(Y, X=matrix(1,nrow(Y),1), betap=matrix(0,ncol(X),ncol(Y)), covp=diag(1,ncol(Y)), Sp=diag(1,ncol(Y)), nburn=100, nbetween=100, nimp=5) {
-
   stopifnot(nrow(Y)==nrow(X), nrow(betap)==ncol(X), ncol(betap)==ncol(Y),nrow(covp)==ncol(covp), nrow(covp)==ncol(Y), nrow(Sp)==ncol(Sp),nrow(Sp)==nrow(covp))
+  betait=matrix(0,nrow(betap),ncol(betap))
+  for (i in 1:nrow(betap)) {
+    for (j in 1:ncol(betap)) betait[i,j]=betap[i,j]
+  }
+  covit=matrix(0,nrow(covp),ncol(covp))
+  for (i in 1:nrow(covp)) {
+    for (j in 1:ncol(covp)) covit[i,j]=covp[i,j]
+  }   
   rngflag=0;
   colnamy<-colnames(Y)
   colnamx<-colnames(X)
@@ -21,7 +28,7 @@ jomo1con<- function(Y, X=matrix(1,nrow(Y),1), betap=matrix(0,ncol(X),ncol(Y)), c
   opost<-matrix(0,nrow(covp),ncol(covp))
   meanobs<-colMeans(Y,na.rm=TRUE)
   for (i in 1:nrow(Y)) for (j in 1:ncol(Y)) if (is.na(Yimp[i,j])) Yimp[i,j]=meanobs[j]
-  .Call("jomo1con", Y, Yimp, Yimp2, X,betap,bpost,covp, opost, nburn, Sp,rngflag, PACKAGE = "jomo")
+  .Call("jomo1con", Y, Yimp, Yimp2, X,betait,bpost,covit, opost, nburn, Sp,rngflag, PACKAGE = "jomo")
   #betapost[,,1]=bpost
   #omegapost[,,1]=opost
   bpost<-matrix(0,nrow(betap),ncol(betap))
@@ -34,7 +41,7 @@ jomo1con<- function(Y, X=matrix(1,nrow(Y),1), betap=matrix(0,ncol(X),ncol(Y)), c
     imp[(i*nrow(X)+1):((i+1)*nrow(X)),(ncol(Y)+1):(ncol(Y)+ncol(X))]=X
     imp[(i*nrow(X)+1):((i+1)*nrow(X)), (ncol(Y)+ncol(X)+1)]=i
     imp[(i*nrow(X)+1):((i+1)*nrow(X)), (ncol(Y)+ncol(X)+2)]=c(1:nrow(Y))
-    .Call("jomo1con", Y, Yimp, Yimp2, X,betap,bpost,covp, opost, nbetween, Sp,rngflag, PACKAGE = "jomo")
+    .Call("jomo1con", Y, Yimp, Yimp2, X,betait,bpost,covit, opost, nbetween, Sp,rngflag, PACKAGE = "jomo")
     betapost[,,(i-1)]=bpost
     omegapost[,,(i-1)]=opost
     bpost<-matrix(0,nrow(betap),ncol(betap))
