@@ -1,6 +1,14 @@
 jomo1mix <-
-  function(Y_con, Y_cat, Y_numcat, X=matrix(1,nrow(Y_cat),1), betap=matrix(0,ncol(X),(ncol(Y_con)+(sum(Y_numcat)-length(Y_numcat)))), covp=diag(1,ncol(betap)), Sp=diag(1,ncol(betap)), nburn=100, nbetween=100, nimp=5, meth="MH") {
-    stopifnot((meth=="MH"|meth=="IW"),nrow(Y_con)==nrow(X), nrow(betap)==ncol(X), ncol(betap)==(ncol(Y_con)+(sum(Y_numcat)-length(Y_numcat))),nrow(covp)==ncol(covp), nrow(covp)==ncol(betap), nrow(Sp)==ncol(Sp),nrow(Sp)==nrow(covp))
+  function(Y_con, Y_cat, Y_numcat, X=matrix(1,nrow(Y_cat),1), betap=matrix(0,ncol(X),(ncol(Y_con)+(sum(Y_numcat)-length(Y_numcat)))), covp=diag(1,ncol(betap)), Sp=diag(1,ncol(betap)), nburn=100, nbetween=100, nimp=5) {
+    stopifnot(nrow(Y_con)==nrow(X), nrow(betap)==ncol(X), ncol(betap)==(ncol(Y_con)+(sum(Y_numcat)-length(Y_numcat))),nrow(covp)==ncol(covp), nrow(covp)==ncol(betap), nrow(Sp)==ncol(Sp),nrow(Sp)==nrow(covp))
+    betait=matrix(0,nrow(betap),ncol(betap))
+    for (i in 1:nrow(betap)) {
+      for (j in 1:ncol(betap)) betait[i,j]=betap[i,j]
+    }
+    covit=matrix(0,nrow(covp),ncol(covp))
+    for (i in 1:nrow(covp)) {
+      for (j in 1:ncol(covp)) covit[i,j]=covp[i,j]
+    }   
     rngflag=0
     colnamycon<-colnames(Y_con)
     colnamycat<-colnames(Y_cat)
@@ -34,12 +42,8 @@ jomo1mix <-
     opost<-matrix(0,nrow(covp),ncol(covp))
     meanobs<-colMeans(Yi,na.rm=TRUE)
     for (i in 1:nrow(Yi)) for (j in 1:ncol(Yi)) if (is.na(Yimp[i,j])) Yimp2[i,j]=meanobs[j]
-    if (meth=="MH") {
-      .Call("jomo1mix", Y, Yimp, Yimp2, Y_cat, X,betap,bpost,covp,opost, nburn, Sp,Y_numcat, ncol(Y_con),rngflag, PACKAGE = "jomo")
-    }
-    if (meth=="IW") {
-      .Call("jomo1mix2", Y, Yimp, Yimp2, Y_cat, X,betap,bpost,covp,opost, nburn, Sp,Y_numcat, ncol(Y_con),rngflag, PACKAGE = "jomo")
-    }
+    .Call("jomo1mix", Y, Yimp, Yimp2, Y_cat, X,betait,bpost,covit,opost, nburn, Sp,Y_numcat, ncol(Y_con),rngflag, PACKAGE = "jomo")
+   
     #betapost[,,1]=bpost
     #omegapost[,,1]=opost
     bpost<-matrix(0,nrow(betap),ncol(betap))
@@ -51,12 +55,7 @@ jomo1mix <-
       imp[(i*nrow(X)+1):((i+1)*nrow(X)),(ncol(Y)+1):(ncol(Y)+ncol(X))]=X
       imp[(i*nrow(X)+1):((i+1)*nrow(X)), (ncol(Y)+ncol(X)+1)]=i
       imp[(i*nrow(X)+1):((i+1)*nrow(X)), (ncol(Y)+ncol(X)+2)]=c(1:nrow(Y))
-      if (meth=="MH") {
-        .Call("jomo1mix", Y, Yimp, Yimp2, Y_cat, X,betap,bpost,covp, opost, nbetween, Sp, Y_numcat, ncol(Y_con),rngflag, PACKAGE = "jomo") 
-      }
-      if (meth=="IW") {
-        .Call("jomo1mix2", Y, Yimp, Yimp2, Y_cat, X,betap,bpost,covp, opost, nbetween, Sp, Y_numcat, ncol(Y_con),rngflag, PACKAGE = "jomo") 
-      }
+        .Call("jomo1mix", Y, Yimp, Yimp2, Y_cat, X,betait,bpost,covit, opost, nbetween, Sp, Y_numcat, ncol(Y_con),rngflag, PACKAGE = "jomo") 
       betapost[,,(i-1)]=bpost
       omegapost[,,(i-1)]=opost
       bpost<-matrix(0,nrow(betap),ncol(betap))
