@@ -1,5 +1,21 @@
 jomo1rancon<- function(Y, X=matrix(1,nrow(Y),1), Z=matrix(1,nrow(Y),1), clus, betap=matrix(0,ncol(X),ncol(Y)), up=matrix(0,nrow(unique(clus)),ncol(Z)*ncol(Y)), covp=diag(1,ncol(Y)), covu=diag(1,ncol(Y)*ncol(Z)), Sp=diag(1,ncol(Y)), Sup=diag(1,ncol(Y)*ncol(Z)), nburn=100, nbetween=100, nimp=5) {
   stopifnot(nrow(Y)==nrow(clus),nrow(Y)==nrow(X), nrow(betap)==ncol(X), ncol(betap)==ncol(Y),nrow(covp)==ncol(covp), nrow(covp)==ncol(Y), nrow(Sp)==ncol(Sp),nrow(Sp)==nrow(covp), nrow(Z)==nrow(Y), ncol(covu)==ncol(up), ncol(up)==ncol(Z)*ncol(Y))
+  betait=matrix(0,nrow(betap),ncol(betap))
+  for (i in 1:nrow(betap)) {
+    for (j in 1:ncol(betap)) betait[i,j]=betap[i,j]
+  }
+  covit=matrix(0,nrow(covp),ncol(covp))
+  for (i in 1:nrow(covp)) {
+    for (j in 1:ncol(covp)) covit[i,j]=covp[i,j]
+  }   
+  uit=matrix(0,nrow(up),ncol(up))
+  for (i in 1:nrow(up)) {
+    for (j in 1:ncol(up)) uit[i,j]=up[i,j]
+  }
+  covuit=matrix(0,nrow(covu),ncol(covu))
+  for (i in 1:nrow(covu)) {
+    for (j in 1:ncol(covu)) covuit[i,j]=covu[i,j]
+  }   
   rngflag=0
   colnamy<-colnames(Y)
   colnamx<-colnames(X)
@@ -33,7 +49,7 @@ jomo1rancon<- function(Y, X=matrix(1,nrow(Y),1), Z=matrix(1,nrow(Y),1), clus, be
   varobs<-apply(Y,2,sd,na.rm=TRUE)
   for (i in 1:nrow(Y)) for (j in 1:ncol(Y)) if (is.na(Yimp[i,j])) Yimp[i,j]=meanobs[j]
   #for (i in 1:nrow(Y)) for (j in 1:ncol(Y)) if (is.na(Yimp[i,j])) Yimp[i,j]=rnorm(1,mean=meanobs[j], sd=0.01)
-  .Call("jomo1rancon", Y, Yimp, Yimp2, X, Z, clus, betap, up, bpost, upost, covp, opost, covu, cpost, nburn, Sp, Sup,rngflag, PACKAGE = "jomo")
+  .Call("jomo1rancon", Y, Yimp, Yimp2, X, Z, clus, betait, uit, bpost, upost, covit, opost, covuit, cpost, nburn, Sp, Sup,rngflag, PACKAGE = "jomo")
   #betapost[,,1]=bpost
   #upostall[,,1]=upost
   #omegapost[,,1]=opost
@@ -52,7 +68,7 @@ jomo1rancon<- function(Y, X=matrix(1,nrow(Y),1), Z=matrix(1,nrow(Y),1), clus, be
     imp[(i*nrow(clus)+1):((i+1)*nrow(clus)), (ncol(Y)+ncol(X)+ncol(Z)+1)]=clus
     imp[(i*nrow(Z)+1):((i+1)*nrow(Z)), (ncol(Y)+ncol(X)+ncol(Z)+2)]=c(1:nrow(Y))
     imp[(i*nrow(Z)+1):((i+1)*nrow(Z)), (ncol(Y)+ncol(X)+ncol(Z)+3)]=i
-    .Call("jomo1rancon", Y, Yimp, Yimp2, X, Z, clus, betap, up, bpost, upost, covp,opost, covu,cpost, nbetween, Sp, Sup,rngflag, PACKAGE = "jomo")
+    .Call("jomo1rancon", Y, Yimp, Yimp2, X, Z, clus, betait, uit, bpost, upost, covit,opost, covuit,cpost, nbetween, Sp, Sup,rngflag, PACKAGE = "jomo")
     betapost[,,(i-1)]=bpost
     upostall[,,(i-1)]=upost
     omegapost[,,(i-1)]=opost
