@@ -284,23 +284,23 @@ for (i=0;i<ns;i++) {
 	for (jj=1;jj<JY;jj++) for (tt=0;tt<jj;tt++) Gammastar[jj+JY*tt]=Gammastar[tt+JY*jj];
 
 	wishart_sample(JY,(nj*a+gamma),Gammastar,invA,help, omegaoo,omegaom,omegamm,fl);
-	u_m=newton_raphson(u_new,precision, dx,eta,JY,nj,allinvomega,invomega,invA,help,help2,gamma,Gammastar);
+	u_m=newton_raphson(u_new,precision, dx,eta,JY,nj,allinvomega,invomega,invA,help,help2);
 	if (u_m!=-9999) {
-		con=-log_f_u(eta, u_m, JY, nj, allinvomega, invomega,  invA,  help,  help2, gamma, Gammastar);
-		deriv2=derive2_f_u(dx,eta, u_m, JY, nj, allinvomega, invomega,  invA,  help,  help2, gamma, Gammastar,con);
+		con=-log_f_u(eta, u_m, JY, nj, allinvomega, invomega,  invA,  help,  help2);
+		deriv2=derive2_f_u(dx,eta, u_m, JY, nj, allinvomega, invomega,  invA,  help,  help2,con);
 		lambda=sqrt(-5/(4*deriv2));
 		u_prop=lambda*t_sample(4,fl)+u_m;
-		con2=exp(log_f_u(eta, u_prop, JY, nj, allinvomega, invomega, invA,help,help2,gamma, Gammastar)-log_f_u(eta, u_new, JY, nj, allinvomega, invomega, invA,help,help2, gamma,Gammastar))*h_u(u_new,u_m,lambda)/h_u(u_prop,u_m,lambda);
+		con2=exp(log_f_u(eta, u_prop, JY, nj, allinvomega, invomega, invA,help,help2)-log_f_u(eta, u_new, JY, nj, allinvomega, invomega, invA,help,help2))*h_u(u_new,u_m,lambda)/h_u(u_prop,u_m,lambda);
 		if ((( double ) unif_rand ( ) )<r8_min(1,con2)) u_new=u_prop;
 		a=u_new;
 	}
 	if (u_m==-9999) {
 		u_m=a;
-		con=-log_f_u(eta, u_m, JY, nj, allinvomega, invomega,  invA,  help,  help2, gamma, Gammastar);
-		deriv2=derive2_log_f_u(dx,eta, u_m, JY, nj, allinvomega, invomega,  invA,  help,  help2, gamma, Gammastar);
+		con=-log_f_u(eta, u_m, JY, nj, allinvomega, invomega,  invA,  help,  help2);
+		deriv2=derive2_log_f_u(dx,eta, u_m, JY, nj, allinvomega, invomega,  invA,  help,  help2);
 		lambda=sqrt(-5/(4*deriv2));
 		u_prop=lambda*t_sample(4,fl)+u_m;
-		con2=exp(log_f_u(eta, u_prop, JY, nj, allinvomega, invomega, invA,help,help2,gamma, Gammastar)-log_f_u(eta, u_new, JY, nj, allinvomega, invomega, invA,help,help2, gamma,Gammastar))*h_u(u_new,u_m,lambda)/h_u(u_prop,u_m,lambda);
+		con2=exp(log_f_u(eta, u_prop, JY, nj, allinvomega, invomega, invA,help,help2)-log_f_u(eta, u_new, JY, nj, allinvomega, invomega, invA,help,help2))*h_u(u_new,u_m,lambda)/h_u(u_prop,u_m,lambda);
 		if ((( double ) unif_rand ( ) )<r8_min(1,con2)) u_new=u_prop;
 		a=u_new;
 
