@@ -14,9 +14,9 @@ int indic=0,i,j,k, IY,JY, IX, JX, Io, Jo, Ib, Jb, ns, nmiss=0,t, countm=0, count
 int Iu, Ju, IZ, JZ, nj,c, fl,currncat;
 SEXP RdimY, RdimX, Rdimo, Rdimb, RdimZ, Rdimu;
 double *betaX, *Yobs, *Ymiss, *mumiss, *omegadrawmiss, *betamiss, *betaobs, *omegaoo, *omegaom, *omegamo, *omegamm, *invomega, *invomega2, *help, *help2, *help3, *imp, *zi;
-double *sumzy, *incrzz, *incrzy, *mu, *mu2, *newbeta, *newomega, *sumzi, *yi, *invomega3, *help4, *help5, *help6, *missing, *fixomega,*resid;
+double *sumzy, *incrzz, *incrzy, *mu, *mu2, *newbeta, *newomega, *sumzi, *yi, *invomega3, *help4, *help5, *help6, *missing, *fixomega,*resid, sdom, meanom, detom,logLH, newlogLH;
 double maxx,maxim,maxim2, *sumxy, *sumxi, *uj, *xi, *ziu, *incrxx, *incrxy, *newu, *mu3,*mu4, *help7, *help8, *help9, *invomega4, *newomega2,a, *clusnum;
-double *cumclus, *allinvomega,gamma,eta,dx,u_new,precision, *invgamma, *invA, *Gammapr, *Gammastar,u_m,con,con2,deriv2,u_prop, lambda;
+double *cumclus, *allinvomega,gamma,eta,dx,u_new,precision, *invgamma, *invA, *Gammapr, *Gammastar,u_m,con,con2,deriv2,u_prop, lambda, aj;
 
 /* Protecting R objects from garbage collection and saving matrices dimensions*/ 
 
@@ -129,7 +129,7 @@ Gammastar= (double * )  R_alloc ( JY * JY , sizeof(double) );
 
 /* Some initializations */
 
-gamma=JY+1;
+gamma=JY;
 //a=JY+1;
 eta=JY;
 dx=0.001;
@@ -137,10 +137,12 @@ u_new=log(a+JY);
 precision=0.001;
 GetRNGstate();
 
+
 for (i=0;i<JY*JY;i++) Gammapr[i]=REAL(Sp)[i];
 r8mat_pofac(JY,Gammapr,help,1);
 r8mat_poinv(JY, help, invgamma);
 for (jj=1;jj<JY;jj++) for (tt=0;tt<jj;tt++) invgamma[jj+JY*tt]=invgamma[tt+JY*jj];
+for (t=0;t<JY*JY;t++) invA[t]=REAL(Sp)[t];
 
 for (j=0; j<IY; j++) {
 	missing[j]=0;
@@ -161,11 +163,10 @@ for (i=0;i<nj;i++) {
 	}
 	cumclus[i+1]=cumclus[i]+clusnum[i];
 }
-cumclus[1]=clusnum[1];
+
 r8mat_copy_new(IY, JY, REAL(Yimp2), imp);
 for (i=0;i<Ib*Jb;i++) REAL(betapost)[i]=0;
 for (i=0;i<JY*JY;i++) fixomega[i]=0;
-for (i=0;i<JY*JY;i++) newomega[i]=REAL(omega)[i];
 pos=ncon;
 for (i=0;i<ncat;i++) {
 	for (j=0;j<(INTEGER(Y_numcat)[i]-1);j++) {
@@ -250,18 +251,14 @@ for (i=0;i<ns;i++) {
 							counto++;
 						}
 					}
-				}
+				}	
+
 				countm=0;
 				counto=0;
 				r8mat_pofac((JY-currncat),help4, help6,1);
 				r8mat_poinv((JY-currncat),help6, invomega);
-			
 				for (jj=1;jj<(JY-currncat);jj++) for (tt=0;tt<jj;tt++) invomega[jj+(JY-currncat)*tt]=invomega[tt+(JY-currncat)*jj];
-			
-
 				r8mat_mm_new((JY-currncat),(JY-currncat),currncat,invomega,help5, help2);
-				
-
 				r8mat_mm_new(1,(JY-currncat),currncat,help,help2, mumiss);
 				r8mat_add(currncat,1,betaX,mumiss);
 				r8mat_mtm_new(currncat,(JY-currncat),currncat,help2,help5, omegadrawmiss);
@@ -269,20 +266,16 @@ for (i=0;i<ns;i++) {
 				for (k=0;k<currncat;k++) {
 					omegadrawmiss[k+k*currncat]=omegadrawmiss[k+k*currncat]+1;
 					for (kk=0;kk<currncat;kk++) if (k!=kk) omegadrawmiss[k+kk*currncat]=omegadrawmiss[k+kk*currncat]+0.5;
-					
 				}
 				r8mat_pofac(currncat,omegadrawmiss, omegamm,2);
 				flag=0;
 				kk=0;
-	
 
 				if (REAL(Y)[t+(ncon+j)*IY]==INTEGER(Y_numcat)[j]) {
 					while ((flag==0)&(kk<10000)) {
 						r8vec_multinormal_sample((INTEGER(Y_numcat)[j]-1), mumiss,omegamm, newbeta,mu4,fl);
 						maxim=maxvec((INTEGER(Y_numcat)[j]-1),newbeta);
-						if (maxim<0) {
-
-						
+						if (maxim<0) {						
 							for (k=0;k<(INTEGER(Y_numcat)[j]-1);k++) imp[t+(k+pos)*IY]=newbeta[k];
 							flag=1;
 							indic++;
@@ -345,6 +338,7 @@ for (i=0;i<ns;i++) {
 	r8mat_mm_new(JY*JX,JY*JX,1,invomega2,sumxy,mu);
 	r8mat_pofac(JY * JX,invomega2,help3,6);
 	r8vec_multinormal_sample(JY*JX, mu,help3, REAL(beta),newbeta,fl);
+	
 	r8mat_add(Ib,Jb,REAL(beta),REAL(betapost));
 
 	//Updating random effects
@@ -428,6 +422,8 @@ for (i=0;i<ns;i++) {
 
 	//Updating omega
 
+	//Updating scale matrix of inverse Wishart
+	
 	for (t=0;t<JY*JY;t++) help[t]=0;
 	for (c=0;c<nj;c++) {
 		for (t=0;t<JY;t++) {
@@ -437,87 +433,105 @@ for (i=0;i<ns;i++) {
 		}
 	}
 	r8mat_add(JY,JY,invgamma,help);
-	r8mat_pofac(JY,help,help7,12);
+	r8mat_pofac(JY,help,help7,10);
 	r8mat_poinv(JY, help7, Gammastar);
 	for (jj=1;jj<JY;jj++) for (tt=0;tt<jj;tt++) Gammastar[jj+JY*tt]=Gammastar[tt+JY*jj];
 
 	wishart_sample(JY,(nj*a+gamma),Gammastar,invA,help, omegaoo,omegaom,omegamm,fl);
-	
-	u_m=newton_raphson(u_new,precision, dx,eta,JY,nj,allinvomega,invomega,invA,help,help2,gamma,Gammastar);
+
+		u_m=newton_raphson(u_new,precision, dx,eta,JY,nj,allinvomega,invomega,invA,help,help2);
 	if (u_m!=-9999) {
-		con=-log_f_u(eta, u_m, JY, nj, allinvomega, invomega,  invA,  help,  help2, gamma, Gammastar);
-		deriv2=derive2_f_u(dx,eta, u_m, JY, nj, allinvomega, invomega,  invA,  help,  help2, gamma, Gammastar,con);	
+		con=-log_f_u(eta, u_m, JY, nj, allinvomega, invomega,  invA,  help,  help2);
+		deriv2=derive2_f_u(dx,eta, u_m, JY, nj, allinvomega, invomega,  invA,  help,  help2,con);
 		lambda=sqrt(-5/(4*deriv2));
-		u_prop=lambda*t_sample(4, fl)+u_m;
-		con2=exp(log_f_u(eta, u_prop, JY, nj, allinvomega, invomega, invA,help,help2,gamma, Gammastar)-log_f_u(eta, u_new, JY, nj, allinvomega, invomega, invA,help,help2, gamma,Gammastar))*h_u(u_new,u_m,lambda)/h_u(u_prop,u_m,lambda);
+		u_prop=lambda*t_sample(4,fl)+u_m;
+		con2=exp(log_f_u(eta, u_prop, JY, nj, allinvomega, invomega, invA,help,help2)-log_f_u(eta, u_new, JY, nj, allinvomega, invomega, invA,help,help2))*h_u(u_new,u_m,lambda)/h_u(u_prop,u_m,lambda);
 		if ((( double ) unif_rand ( ) )<r8_min(1,con2)) u_new=u_prop;
 		a=u_new;
 	}
 	if (u_m==-9999) {
 		u_m=a;
-		con=-log_f_u(eta, u_m, JY, nj, allinvomega, invomega,  invA,  help,  help2, gamma, Gammastar);
-		deriv2=derive2_log_f_u(dx,eta, u_m, JY, nj, allinvomega, invomega,  invA,  help,  help2, gamma, Gammastar);
+		con=-log_f_u(eta, u_m, JY, nj, allinvomega, invomega,  invA,  help,  help2);
+		deriv2=derive2_log_f_u(dx,eta, u_m, JY, nj, allinvomega, invomega,  invA,  help,  help2);
 		lambda=sqrt(-5/(4*deriv2));
 		u_prop=lambda*t_sample(4,fl)+u_m;
-		con2=exp(log_f_u(eta, u_prop, JY, nj, allinvomega, invomega, invA,help,help2,gamma, Gammastar)-log_f_u(eta, u_new, JY, nj, allinvomega, invomega, invA,help,help2, gamma,Gammastar))*h_u(u_new,u_m,lambda)/h_u(u_prop,u_m,lambda);
+		con2=exp(log_f_u(eta, u_prop, JY, nj, allinvomega, invomega, invA,help,help2)-log_f_u(eta, u_new, JY, nj, allinvomega, invomega, invA,help,help2))*h_u(u_new,u_m,lambda)/h_u(u_prop,u_m,lambda);
 		if ((( double ) unif_rand ( ) )<r8_min(1,con2)) u_new=u_prop;
 		a=u_new;
-}
+
+	}
 	if (a<JY) a=JY;
 	if (isnan(a)) a=JY;
 	u_new=a;	
+	
+	//Updating matrices
+	
 	for (c=0;c<nj;c++) {	
-		for (j=0;j<JY*JY;j++) newomega2[j]=0;
-		for (j=0;j<JY*JY;j++) mu4[j]=0;
-		for (j=cumclus[c];j<cumclus[c+1];j++) {
-			for (t=0;t<JY*JZ*JY;t++) zi[t]=0;
-			for (t=0;t<JY*JX*JY;t++) xi[t]=0; 
-			for (t=0;t<JY;t++) {
-				yi[t]=imp[j+t*IY];
-				for (k=0;k<JX;k++) {
-					xi[t+(k+t*JX)*JY]=REAL(X)[j+IY*k];
-				}
-				for (k=0;k<JZ;k++) {
-					zi[t+(k+t*JZ)*JY]=REAL(Z)[j+IY*k];
-					uj[k+t*JZ]=REAL(u)[c+nj*(k+t*JZ)];
-				}
+	aj=a+clusnum[c];
+	for (kk=0;kk<JY;kk++) {
+		for (tt=0;tt<JY;tt++) {
+			newomega[kk+tt*JY]=REAL(omega)[(c*JY+kk)+tt*(JY*nj)];
 			}
-			r8mat_mm_new(JY,JY*JZ,1,zi,uj,ziu);
-			r8mat_divide(JY,1,-1,ziu);
-			r8mat_add(JY,1,ziu,yi);
-			r8mat_mm_new(JY,JY*JX,1,xi,REAL(beta),ziu);
-			r8mat_divide(JY,1,-1,ziu);
-			r8mat_add(JY,1,ziu,yi);
-			r8mat_mmt_new(JY,1,JY,yi,yi,help);
-			r8mat_add(JY,JY,help,mu4);
 		}
-		r8mat_add(JY,JY,invA,mu4);
-		r8mat_pofac(JY,mu4,help,13);
-		r8mat_poinv(JY, help,invomega);
-		for (jj=1;jj<JY;jj++) for (tt=0;tt<jj;tt++) invomega[jj+JY*tt]=invomega[tt+JY*jj];
-		kk=0;
-		flag=0;
-		while ((flag==0)&(kk<100)) {	
-			wishart_sample(JY,clusnum[c]+a,invomega,newomega2,help,omegaoo,omegaom,omegamm,fl);
-			r8mat_pofac(JY,newomega2,help,14);
-			r8mat_poinv(JY, help,invomega2);
-			for (jj=1;jj<JY;jj++) for (tt=0;tt<jj;tt++) invomega2[jj+JY*tt]=invomega2[tt+JY*jj];
-			for (k=0;k<JY;k++) {
-				for (j=0;j<JY;j++) {
-					if (fixomega[k+JY*j]==1) {
-						if (k==j) invomega2[k+JY*j]=1;
-						else invomega2[k+JY*j]=0.5;
-					}
-				}
-			}
-			flag=checkposdef(JY,invomega2,help,help5);
-			kk++;
+	flag=0;
+	r8mat_pofac(JY,newomega,help,6);
+	detom=r8mat_podet(JY, help);
 
-		}
-		if (flag!=0) {	
-			for(k=0;k<JY;k++)  for(j=0;j<JY;j++)  REAL(omega)[(c*JY+j)+k*(JY*nj)]=invomega2[j+JY*k];
+	r8mat_poinv(JY, help,invomega);
+	for (jj=1;jj<JY;jj++) for (tt=0;tt<jj;tt++) invomega[jj+JY*tt]=invomega[tt+JY*jj];
+	logLH=0;
+	for (t=0;t<JY*JY;t++) help5[t]=0;
+	for (t=cumclus[c];t<cumclus[c+1];t++) {
+		for (jj=0;jj<JY;jj++) help4[jj]=resid[t+jj*IY];
+			r8mat_mmt_new(JY,1,JY,help4,help4,help7);
+			r8mat_add(JY,JY,help7, help5);
+	}
+	r8mat_add(JY,JY,invA, help5);
+
+	r8mat_mm_new(JY,JY,JY,help5,invomega,help6);
+	for (t=0;t<JY;t++) logLH=logLH+help6[t+t*JY];
+	logLH=logLH*(-0.5)-(aj+JY+1)*log(detom)/2;
+	for (j=0;j<JY;j++) {
+		for (k=0;k<JY;k++) {
+			if ((fixomega[j+JY*k]==0)&(j<=k)) {
+				if (j==k) {
+					sdom=REAL(omega)[(c*JY+j)+k*(JY*nj)]*sqrt(11.6/clusnum[c]);
+					meanom=REAL(omega)[(c*JY+j)+k*(JY*nj)];
+				}
+				else {
+					sdom=0.1*sqrt(REAL(omega)[(c*JY+j)+j*(JY*nj)]*REAL(omega)[(c*JY+k)+k*(JY*nj)]);
+					meanom=REAL(omega)[(c*JY+j)+k*(JY*nj)];
+				}
+				kk=0;
+				while ((flag==0)&(kk<100)) {
+					newomega[j+JY*k]=r8_normal_sample(meanom,sdom,fl);
+					newomega[k+JY*j]=newomega[j+JY*k];
+					flag=checkposdef(JY,newomega,help,help6);
+					kk++;
+				}
+				r8mat_pofac(JY,newomega,help,7);
+				detom=r8mat_podet(JY, help);
+				r8mat_poinv(JY, help,invomega);
+				for (jj=1;jj<JY;jj++) for (tt=0;tt<jj;tt++) invomega[jj+JY*tt]=invomega[tt+JY*jj];
+				newlogLH=0;
+				r8mat_mm_new(JY,JY,JY,help5,invomega,help6);
+				for (t=0;t<JY;t++) newlogLH=newlogLH+help6[t+t*JY];
+				newlogLH=newlogLH*(-0.5)-(aj+JY+1)*log(detom)/2;
+				if (((( double ) unif_rand ( ))<exp(newlogLH-logLH))&(flag==1)) {
+					REAL(omega)[(c*JY+j)+k*(JY*nj)]=newomega[j+JY*k];
+					REAL(omega)[(c*JY+k)+j*(JY*nj)]=newomega[k+JY*j];
+					logLH=newlogLH;
+				}
+				else {
+					newomega[j+JY*k]=REAL(omega)[(c*JY+j)+k*(JY*nj)];
+					newomega[k+JY*j]=REAL(omega)[(c*JY+k)+j*(JY*nj)];
+				}
+				flag=0;
+			}
 		}
 	}
+	}
+
 	r8mat_add(Io,Jo,REAL(omega),REAL(omegapost));
 
 	//imputing missing values
@@ -622,11 +636,11 @@ for(i=0;i<IY;i++)  {
 	}
 
 }
-REAL(a_start)[0]=a;
 r8mat_divide(Ib,Jb,ns,REAL(betapost));
 r8mat_divide(Iu,Ju,ns,REAL(upost));
 r8mat_divide(JY*JZ,JY*JZ,ns,REAL(covupost));
 r8mat_divide(JY,nj*JY,ns,REAL(omegapost));
+REAL(a_start)[0]=a;
 PutRNGstate();
 UNPROTECT(28);
 return R_NilValue;
